<?php
namespace SupplyTek\Services;

use Doctrine\ORM\EntityManager;
use SupplyTek\Libraries\NewCode;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

/**
 * Class that handle insert, update and delete from cart
 * 
 * @author Danilo D. Vaz
 */
class CartService
{
    protected $connection;
    protected $newCodeSC;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param EntityManager $entityManager Object to manage database connection and queries
     * @param NewCode $newCodeSC Object to generate new code without commiting
     * 
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function __construct(EntityManager $entityManager, NewCode $newCodeSC)
    {
        $this->connection = $entityManager->getConnection();
        $this->newCodeSC = $newCodeSC;
    }
    
    /**
     * Function that begin database transaction
     * 
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function beginTransaction () {
        $this->connection->beginTransaction();
    }
    
    /**
     * Function that commit database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function commit () {
        $this->connection->commit();
    }
    
    /**
     * Function that rollback database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function rollback () {
        $this->connection->rollback();
    }

    /**
     * Insert a new cart
     * 
     * Generate a sequential code and insert a new cart
     * 
     * @param string $operatorId String with operator id
     * @param string $unitId String with unit id
     * @param string $productId String with product id
     * @param string $orgId String with organization id
     *
     * @return string
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function insertCart ($operatorId, $unitId, $productId, $orgId) 
    {
        $cartId = $this->newCodeSC->getCodeByTableName('CARTSUPPLY', $orgId, 1, '', true);
        
        $sql = "INSERT INTO CARTSUPPLY 
                (
                    NRCARTSUPPLY,
                    CDFILIAL,
                    CDOPERADOR,
                    CDFORNECED,
                    IDTIPOCART,
                    IDINAUGURA,
                    IDSITUCOMPRA,
                    DTINCLUSAO
                )
                (
                    SELECT DISTINCT
                        :CART_ID,
                        :UNIT_ID,
                        :OPERATOR_ID,
                        CDFORNECED,
                        'P',
                        'N',
                        'A',
                        SYSDATE
                     FROM V_CATALOG
                    WHERE NRSEQ = :ID
                )";
                
        $params = array(
            "CART_ID" => $cartId,
            "ID" => $productId,
            "UNIT_ID" => $unitId,
            "OPERATOR_ID" => $operatorId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        
        return $cartId;
    }
    
    /**
     * Update cart
     * 
     * @param string $cartId String with cart id
     * @param string $purchaseSituation String with purchase situation
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function updateCart ($cartId, $purchaseSituation)
    {
        $sql = "UPDATE CARTSUPPLY
                   SET IDSITUCOMPRA = :PURCHASE_SITUATION
                 WHERE NRCARTSUPPLY = :CART_ID";
                
        $params = array(
            "CART_ID" => $cartId,
            "PURCHASE_SITUATION" => $purchaseSituation
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Insert a new item in cart
     * 
     * Generate a sequential code and insert a new item in cart
     * 
     * @param string $cartId String with cart id
     * @param string $operatorId String with operator id
     * @param string $unitId String with unit id
     * @param array $product Array with product information
     * @param string $orgId String with organization id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function insertItemCart ($cartId, $operatorId, $unitId, $product, $orgId)
    {
        $cartItemId = $this->newCodeSC->getCodeByTableName('PRODCART', $orgId, 1, '', true);
        
        $sql = "INSERT INTO PRODCART 
                (
                    NRPRODCART,
                    NRCARTSUPPLY,
                    NRSEQCATALOG,
                    CDPRODUTO,
                    QTPRODUTO,
                    VRPRECPROD,
                    CDMARCA,
                    CDCATEGORIA,
                    CDSUBCATEGORIA,
                    CDCOMPRADOR,
                    DTINCLUSAO
                )
                (
                    SELECT
                        :CART_ITEM_ID,
                        :CART_ID,
                        V_CATALOG.NRSEQ,
                        ITEMCOLE.CDPRODUTO,
                        :AMOUNT,
                        ITEMCOLE.VRPRECUNIITM,
                        ITEMCOLE.CDMARCA,
                        V_CATALOG.CDCATEGORIA,
                        V_CATALOG.CDSUBCATEGORIA,
                        V_CATALOG.CDCOMPRADOR,
                        SYSDATE
                      FROM V_CATALOG, ITEMCOLE, PRODUTO, ACESSOSUPPLY, FILIAL
                     WHERE V_CATALOG.NRSEQ = :ID
                       AND NVL(PRODUTO.IDTAMANHO, 'UNICO') = :SIZES
                       AND ROUND(NVL(ITEMCOLE.VRPRECUNIITM, 0), 2) = :PRICE
                       AND ACESSOSUPPLY.CDOPERADOR = :OPERATOR_ID
                       AND FILIAL.CDFILIAL = :UNIT_ID
                       AND ACESSOSUPPLY.CDFILIAL = FILIAL.CDFILIAL
                       AND V_CATALOG.CDREGIAO = FILIAL.CDREGIAO
                       AND V_CATALOG.CDFILCOT = ITEMCOLE.CDFILIAL
                       AND V_CATALOG.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO
                       AND V_CATALOG.CDFORNECED = ITEMCOLE.CDFORNECED
                       AND V_CATALOG.CDMARCA = ITEMCOLE.CDMARCA
                       AND V_CATALOG.CDREGIAO = ITEMCOLE.CDREGIAO
                       AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO
                       AND V_CATALOG.CDPRODUTO = DECODE(PRODUTO.IDTAMANHO, '', PRODUTO.CDPRODUTO, PRODUTO.CDPRODESTO)
                )";
        
        $params = array(
            "CART_ITEM_ID" => $cartItemId,
            "CART_ID" => $cartId,
            "AMOUNT" => $product['AMOUNT'],
            "ID" => $product['ID'],
            "SIZES" => $product['SIZE'] ? $product['SIZE']: "UNICO",
            "PRICE" => $product['PRICE'],
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId
        );
        
        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Update cart item
     * 
     * @param string $cartItemId String with cart item id
     * @param string $amount String with amount
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function updateItemCart ($cartItemId, $amount)
    {
        $sql = "UPDATE PRODCART 
                   SET QTPRODUTO = :AMOUNT
                 WHERE NRPRODCART = :CART_ITEM_ID";
                
        $params = array(
            "CART_ITEM_ID" => $cartItemId,
            "AMOUNT" => $amount
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Delete cart item
     * 
     * Can delete just one item or all items from cart
     * 
     * @param string $cartId String with cart id
     * @param string $cartItemId String with cart item id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function deleteItemCart ($cartId, $cartItemId = null)
    {
        $sql = "DELETE FROM PRODCART
                 WHERE (NRCARTSUPPLY = :CART_ID OR :CART_ID IS NULL)
                   AND (NRPRODCART = :CART_ITEM_ID OR :CART_ITEM_ID IS NULL)";
                
        $params = array(
            "CART_ID" => $cartId,
            "CART_ITEM_ID" => $cartItemId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Delete cart
     * 
     * Delete a cart without items
     * 
     * @param string $cartId String with cart id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function deleteCartWithoutItem ($cartId)
    {
        $sql = "DELETE
                  FROM CARTSUPPLY
                 WHERE NRCARTSUPPLY = :CART_ID
                   AND NOT EXISTS (SELECT 1 
                                     FROM PRODCART 
                                    WHERE CARTSUPPLY.NRCARTSUPPLY = PRODCART.NRCARTSUPPLY)";
                
        $params = array(
            "CART_ID" => $cartId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Update supply authorization
     * 
     * @param string $unitDest String with target unit
     * @param string $unitEntry String with launch unit
     * @param string $supplyAuthId String with supply authorization id
     * @param string $operatorId String with operator id
     * @param string $comment String that receives justification for approval
     * @param string $cartId String with cart id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function updateSupplyAuth ($unitDest, $unitEntry, $supplyAuthId, $operatorId, $comment, $cartId)
    {
        $sql = "UPDATE AUTOFORNE
                   SET IDAFCATALOGO = 'S', 
                       CDOPERLANCAF = :OPERATOR_ID, 
                       IDBAIXAAF = NULL,
                       DSMOTVAFALCADA = :DESC_COMMENT,
                       NRCARTSUPPLY = :CART_ID
                 WHERE CDFILDEST  = :UNIT_DEST  
                   AND CDFILLANC  = :UNIT_ENTRY
                   AND NRAUTOFORN = :SUPPLY_AUTH_ID";
                
        $params = array(
            "OPERATOR_ID" => $operatorId,
            "DESC_COMMENT" => $comment,
            "CART_ID" => $cartId,
            "UNIT_DEST" => $unitDest,
            "UNIT_ENTRY" => $unitEntry,
            "SUPPLY_AUTH_ID" => $supplyAuthId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Update supply authorization item
     * 
     * @param string $unitDest String with target unit
     * @param string $unitEntry String with launch unit
     * @param string $supplyAuthId String with supply authorization id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function updateSupplyAuthItem ($unitDest, $unitEntry, $supplyAuthId)
    {
        $sql = "UPDATE ITEMAUTF 
                   SET QTPROGENTORI = QTPROGENTAF 
                 WHERE CDFILDEST  = :UNIT_DEST     
                   AND CDFILLANC  = :UNIT_ENTRY  
                   AND NRAUTOFORN = :SUPPLY_AUTH_ID";
                
        $params = array(
            "UNIT_DEST" => $unitDest,
            "UNIT_ENTRY" => $unitEntry,
            "SUPPLY_AUTH_ID" => $supplyAuthId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Verify if the supplier generate bill to pay
     * 
     * @param string $supplierId string 
     *
     * @return string
     * 
     * @access public
     * @author Tarcisio Junior
     */
    public function verifySupplierGenerateBillToPay ($supplierId)
    {
        $sql = "SELECT IDGERCONTPGT FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDFORNECED" => $supplierId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        $result = $std->fetch();
        return $result;
    }
    
    /**
     * Execute procedure APURACAO_SUPPLYTEK
     * 
     * Generate apuration id and execute procedure
     * 
     * @param string $purchaseCenter String with purchase center
     * @param string $deliveryDate String with delivery date
     * @param string $cartId String with cart id
     * @param string $unitId String with unit id
     * @param string $orgId String with organization id
     *
     * @return string
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function execProcApuracaoSupplytek ($purchaseCenter, $deliveryDate, $cartId, $unitId, $orgId)
    {
        $apurationId = $this->newCodeSC->getCodeByTableName('APURPRECO' . $purchaseCenter, $orgId);

        $procedure = new StoredProcedure($this->connection, 'APURACAO_SUPPLYTEK');
        $procedure->addParam(new Param('P_CDCENTCMP',    Param::PARAM_INPUT, $purchaseCenter, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPUR',    Param::PARAM_INPUT, $apurationId,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTREGA',    Param::PARAM_INPUT, $deliveryDate,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCARTSUPPLY', Param::PARAM_INPUT, $cartId,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $unitId,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, null,           Param::PARAM_TYPE_STR, 200));
        
        $procedure->execute();
        
        return $apurationId;
    }
    
    /**
     * Execute procedure EFETIVAR_AF
     * 
     * @param string $purchaser String with purchaser
     * @param string $date String with date
     * @param string $purchaseCenter String with purchase center
     * @param string $apurationId String with apuration id
     * @param string $unitId String with unit id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function execProcEfetivarAf ($purchaser, $date, $purchaseCenter, $apurationId, $unitId)
    {
        $procedure = new StoredProcedure($this->connection, 'EFETIVAR_AF');
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $purchaser,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTAUTOFORN', Param::PARAM_INPUT, $date,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCENTCMP',  Param::PARAM_INPUT, $purchaseCenter, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPUR',  Param::PARAM_INPUT, $apurationId,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $unitId,         Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    /**
     * Execute procedure FECHAMENTO_AF
     * 
     * @param string $unitDest String with target unit
     * @param string $unitEntry String with launch unit
     * @param string $supplyAuthId String with supply authorization id
     *
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function execProcFechamentoAF ($unitDest, $unitEntry, $supplyAuthId)
    {
        $procedure = new StoredProcedure($this->connection, 'FECHAMENTO_AF');
        
        $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT, $unitDest,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT, $unitEntry,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $supplyAuthId, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    /**
     * Execute procedure EXCLUI_CONTAPAGAR_PREVISTA
     * 
     * @param string $unitDest String with target unit
     * @param string $unitEntry String with launch unit
     * @param string $supplyAuthId String with supply authorization id
     *
     * @return void
     * 
     * @access public
     * @author Tarcisio Junior
     */
    public function execProcExcluiContaPagarPrevista ($unitDest, $unitEntry, $supplyAuthId)
    {
        $procedure = new StoredProcedure($this->connection, 'EXCLUI_CONTAPAGAR_PREVISTA');
        
        $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT, $unitDest,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT, $unitEntry,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $supplyAuthId, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RESULTADO',  Param::PARAM_OUTPUT, null,         Param::PARAM_TYPE_STR, 200));
        
        $procedure->execute();
    }
    
    /**
     * Execute procedure GERA_CONTAPAGAR_PREVISTA
     * 
     * @param string $unitDest String with target unit
     * @param string $unitEntry String with launch unit
     * @param string $supplyAuthId String with supply authorization id
     *
     * @return void
     * 
     * @access public
     * @author Tarcisio Junior
     */
    public function execProcGeraContaPagarPrevista ($unitDest, $unitEntry, $supplyAuthId)
    {
        $procedure = new StoredProcedure($this->connection, 'GERA_CONTAPAGAR_PREVISTA');
        
        $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT, $unitDest,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT, $unitEntry,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT, $supplyAuthId, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
}